@@ -189,6 +189,14 @@ class Label(ttk.Label, mixin.SingleWidget):
     '''
     label: typing.Optional[str] = None
     state_type = var.nothing
+    wimg: typing.Optional[tk.Image]
+    '''The image object, if defined.
+
+    This avoids the issue with image objects being garbage collected, while keeping a reference for possible manipulation.
+
+    Note:
+        This is a read-only reference, it must be manipulated using `set_image`.
+    '''
 
     def __init__(self, parent, *, label: typing.Optional[str] = None, image: typing.Optional[tk.Image] = None, **kwargs):
         # TODO: Support `anchor`, with `model.CP`
@@ -202,20 +210,92 @@ class Label(ttk.Label, mixin.SingleWidget):
             self.set_label(chosen_label)
         if image:
             self.set_image(image)
+        else:
+            self.wimg = None
+        self.binding('<Button-1>', self.invoke_onClick)
+        if __debug__:
+            if 'textvariable' in kwargs:
+                warnings.warn(f'{self}: To use a variable as label, see LabelStateful', stacklevel=2)
+
+    def invoke_onClick(self, event=None):
+        ''''''  # Internal, do not document
+        self.onClick()
+
+    def set_image(self, image: typing.Optional[tk.Image]) -> None:
+        '''Change the label image.
+
+        Args:
+            image: The image object to set, or `None` to remove it.
+        '''
+        self.wimg = image  # Save a reference to avoid garbage collection
+        self['image'] = self.wimg
+
+    def set_label(self, label: str) -> None:
+        '''Change the label text.'''
+        self['text'] = label
+
+    # TODO: Pass the event object?
+    # TODO: Not needed on labels
+    def onClick(self):
+        """Callback to be executed when clicking this widget.
+
+        Defaults to doing nothing.
+
+        Available for subclass redefinition.
+        """
+        pass
+
+
+class LabelStateful(ttk.Label, mixin.SingleWidget):
+    '''A stateful `Label`, where the text is controlled by a variable.
+
+    This is an alternative `Label`, where the text is controlled by a variable.
+    It's an alternative to manually calling `Label.set_label`, or a read-only
+    `Entry`.
+
+    The image is optional.
+
+    There is no Python documentation, see ``Tk`` `ttk.Label <https://www.tcl.tk/man/tcl/TkCmd/ttk_label.html>`_ documentation.
+
+    Args:
+        variable: Use an externally defined variable, instead of creating a new one specific for this widget.
+        parent: The parent widget. Can be a `RootWindow` or another `mixin.ContainerWidget`.
+    '''
+    state_type = var.String
+    wimg: typing.Optional[tk.Image]
+    '''The image object, if defined.
+
+    This avoids the issue with image objects being garbage collected, while keeping a reference for possible manipulation.
+
+    Note:
+        This is a read-only reference, it must be manipulated using `set_image`.
+    '''
+
+    def __init__(self, parent, *, variable: typing.Optional[var.String] = None, image: typing.Optional[tk.Image] = None, **kwargs):
+        self.init_single(variable=variable)
+        kwargs.update({
+            'textvariable': self.variable,
+        })
+        kwargs.pop('text', None)  # Ignore the given text
+        super().__init__(parent, **kwargs)  # ttk.Label
+        if image:
+            self.set_image(image)
+        else:
+            self.wimg = None
         self.binding('<Button-1>', self.invoke_onClick)
 
     def invoke_onClick(self, event=None):
         ''''''  # Internal, do not document
         self.onClick()
 
-    def set_image(self, image: tk.Image) -> None:
-        '''Change the label image.'''
-        self._image = image  # Save a reference to avoid garbage collection
-        self['image'] = self._image
+    def set_image(self, image: typing.Optional[tk.Image]) -> None:
+        '''Change the label image.
 
-    def set_label(self, label: str) -> None:
-        '''Change the label text.'''
-        self['text'] = label
+        Args:
+            image: The image object to set, or `None` to remove it.
+        '''
+        self.wimg = image  # Save a reference to avoid garbage collection
+        self['image'] = self.wimg
 
     # TODO: Pass the event object?
     # TODO: Not needed on labels
@@ -488,6 +568,7 @@ class FrameStateful(ttk.LabelFrame, mixin.ContainerWidget):
             When `None`, the value is not changed at start.
             Defaults to starting enabled, unless ``cvariable`` is given, for
             which the value is not changed.
+        cstateArgs: Extra arguments for the `cstate` widget, `cstate_widget`.
 
         parent: The parent widget. Can be a `RootWindow` or another `mixin.ContainerWidget`.
 
@@ -518,13 +599,19 @@ class FrameStateful(ttk.LabelFrame, mixin.ContainerWidget):
 
     def __init__(self, parent, *args, label: typing.Optional[str] = None, labelAnchor: model.CP = model.CP.default,
                  cvariable: typing.Optional[var.Boolean] = None, cvariableDefault: typing.Optional[bool] = None,
+                 cstateArgs: typing.Optional[typing.Mapping[str, typing.Any]] = None,
                  **kwargs):
         # Create the checkbox widget
         chosen_label = self.label or label
         if chosen_label is None:
             raise ValueError('{self!r}: Missing required label')
-        # TODO: Support `cstateArgs`, to append arbitrary arguments?
-        cstate_widget = self.__class__.__w_cstate(parent, label=chosen_label, readonly=False, variable=cvariable)
+        cstateArgs = dict(cstateArgs or {})
+        cstateArgs.update({  # Override cstate arguments
+            'variable': cvariable,
+            'label': chosen_label,
+            'readonly': False,
+        })
+        cstate_widget = self.__class__.__w_cstate(parent, **cstateArgs)
         assert isinstance(cstate_widget.variable, var.Boolean), f'{self!r} checkbox widget is not a simple boolean'
         self.cstate = cstate_widget.variable
         # Usual Initialization
@@ -542,17 +629,23 @@ class FrameStateful(ttk.LabelFrame, mixin.ContainerWidget):
         if cvariableDefault is not None:
             self.cstate.set(cvariableDefault)
 
-    def state_get(self, *args, **kwargs) -> model.WState[bool, typing.Any]:
+    def state_get(self, *args, vid_upstream: typing.Optional[typing.Set[str]] = None, **kwargs) -> model.WState[bool, typing.Any]:
         ''''''  # Do not document
+        cid = fn.vname(self.cstate)
+        if vid_upstream and cid in vid_upstream:
+            cstate = None
+        else:
+            cstate = self.cstate.get()
         return model.WState(
-            self.cstate.get(),
-            super().state_get(*args, **kwargs),
+            cstate,
+            super().state_get(*args, vid_upstream=vid_upstream, **kwargs),
         )
 
-    def state_set(self, state: model.WState, *args, **kwargs):
+    def state_set(self, state: model.WState[bool, typing.Any], *args, **kwargs):
         ''''''  # Do not document
-        assert isinstance(state.state, bool), f'Invalid WState: {state!r}'
-        self.cstate.set(state.state)
+        if state.state is not None:
+            assert isinstance(state.state, bool), f'Invalid WState: {state!r}'
+            self.cstate.set(state.state)
         super().state_set(state.substate, *args, **kwargs)
 
     def set_gui_state(self, state: typing.Optional[model.GuiState] = None, **kwargs) -> None:
@@ -605,6 +698,8 @@ class EntryMultiline(tk.Text, mixin.SingleWidget):
         practical effects.
     '''
     state_type = var.String
+    _bindings_tag: typing.MutableMapping[typing.Tuple[str, str], model.BindingTag]
+    '''Store all widget `BindingTag` objects, keyed by ``(tag, name)`` (see `binding_tag`).'''
 
     @dataclass
     class Style(model.WStyle):
@@ -634,17 +729,20 @@ class EntryMultiline(tk.Text, mixin.SingleWidget):
         # GUI State Tracker
         # - Since this is not a `ttk.Widget`, this need to be emulated
         self.__gstate: model.GuiState = model.GuiState(enabled=True, readonly=readonly)
+        # - Set the internal readonly state out-of-the-box
+        self.set_gui_state(readonly=readonly, _internal=True)
         assert self.variable is not None, f'{self!r}: Missing variable'
         self.wstate = getattr(self.variable, '_default', '')  # Set the default (before the trace)
         # State Tracker
         self.trace(self._varChanged, trace_name=f'__:{__name__}')
 
         # Bindings
+        self._bindings_tag = {}
         # TODO: Investigate changing the cursor on mouseover
-        # TODO: Support `BindingTag`(`self.binding_tag`)?
-        self.tag_bind('a', '<Button-1>', self._onClickTag, add=True)
+        self.binding_tag('a', '<Button-1>', self._onClickTag)
         if readonly:
-            self.binding('<Double-Button-1>', lambda e: 'break')  # Disable double click
+            # Disable Double-Click event, when readonly
+            self.binding('<Double-Button-1>', fn.binding_disable)
 
     def _varChanged(self, var, etype):
         assert etype == 'write'
@@ -660,7 +758,7 @@ class EntryMultiline(tk.Text, mixin.SingleWidget):
             # Reset styles
             self.style_reset()
             # Add the current state
-            # TODO: Save the TextElement somewhere in this object, with `data`?
+            # TODO: Save the parsed LTML state somewhere in this object, with `data`?
             for te in parser.parse_LTML(vs):
                 assert isinstance(te, model.TextElement)
                 self.insert(tk.END, te.text, te.atags)
@@ -676,8 +774,8 @@ class EntryMultiline(tk.Text, mixin.SingleWidget):
         ''''''  # Do not document
         if state is None:
             state = model.GuiState(**kwargs)
-        # if __debug__:
-        #     logger.debug('State < %r', state)
+        if __debug__:
+            logger.debug('State < %r', state)
         # Adjust current state
         for sname, svalue in state.items():
             assert sname != '_internal'  # Should be impossible...
@@ -726,6 +824,31 @@ class EntryMultiline(tk.Text, mixin.SingleWidget):
         finally:
             self.set_gui_state(readonly=True, _internal=True)
 
+    def binding_tag(self, tag: str, sequence: str, *args, key: str = None, immediate: bool = True, **kwargs) -> model.BindingTag:
+        '''Binds a sequence to a tag.
+
+        Stores all widget tag bindings on a per-instance dictionary, for later
+        usage. Note that all dictionary keys must be different. For the same
+        bindings on a single widget tag, this requires passing the ``key``
+        argument.
+
+        See the ``Tk`` `tag bind <https://www.tcl.tk/man/tcl/TkCmd/text.html#M166>`_ documentation.
+
+        Args:
+            key: Optional. Defaults to the ``sequence`` itself. Useful to
+                support multiple bindings on the same sequence, for the same tag.
+
+        All other arguments are passed to `model.BindingTag` object.
+        '''
+        name = (tag, key or sequence)
+        if name in self._bindings_tag:
+            raise ValueError(f'Repeated binding for "{sequence}" in {self!r}(tag "{tag}"). Change the "key" parameter.')
+        if __debug__:
+            if len(tag) == 0 or tag[0] == '<':
+                warnings.warn(f'{self}: binding_tag requires tag[{tag}] and sequence[{sequence}], by this order', stacklevel=2)
+        self._bindings_tag[name] = model.BindingTag(self, tag, sequence, *args, immediate=immediate, **kwargs)
+        return self._bindings_tag[name]
+
     def _style_a(self, tag: str = 'a', *, visited: bool) -> None:
         fg = self.wstyle.colour_link_visited if visited else self.wstyle.colour_link_normal
         self.tag_configure(tag, foreground=fg, underline=True)
@@ -781,7 +904,7 @@ class EntryMultiline(tk.Text, mixin.SingleWidget):
         self.onClickTag(tag, tagi, tags_other)
 
     def onClickTag(self, tag: str, tag_index: str, tags_other: typing.Sequence[str]) -> None:
-        """Callback to be called when clicking ``a`` tags in this widget.
+        '''Callback to be called when clicking ``a`` tags in this widget.
 
         Defaults to doing nothing.
 
@@ -791,9 +914,32 @@ class EntryMultiline(tk.Text, mixin.SingleWidget):
             tag: The main tag type. In this case, it's always ``a``.
             tag_index: The tag index. See `LTML <parser.LTML>` Automatic Counter tags.
             tags_other: List of extra tags attached to the anchor. Might be empty.
-        """
+        '''
         pass
 
+    def wstateLTML(self) -> typing.Generator[model.TextElement, None, None]:
+        '''Return the parsed LTML state, as a generator of `model.TextElement`.
+
+        See Also:
+            - `wstate`: Return the LTML string.
+            - `wstateText`: Return the underlying text, without any tags.
+        '''
+        # TODO: Save the parsed LTML state somewhere in this object, with `data`?
+        yield from parser.parse_LTML(self.wstate)
+
+    def wstateText(self) -> str:
+        '''Strip all LTML tags and return the underlying text.
+
+        See Also:
+            - `wstate`: Return the LTML string.
+            - `wstateLTML`: Generate a list of LTML parsed objects.
+        '''
+        # TODO: Save the parsed LTML state somewhere in this object, with `data`?
+        return ''.join(
+            te.text
+            for te in parser.parse_LTML(self.wstate)
+        )
+
 
 class Notebook(ttk.Notebook, mixin.ContainerWidget):
     '''A tabbed interface to hold other containers.
@@ -877,6 +1023,23 @@ class Notebook(ttk.Notebook, mixin.ContainerWidget):
         '''
         raise NotImplementedError
 
+    def wtab(self, idx: str) -> mixin.ContainerWidget:
+        '''Get the tab widget by identifier.
+
+        This is just an helper function to get the correct widget value out of
+        `wtabs`.
+
+        Args:
+            idx: The tab identifier.
+
+        Note:
+            In debug mode, this will fail if called with the wrong identifier.
+            This check is skipped for performance on optimized mode.
+        '''
+        wtab = self.wtabs.get(idx, None)
+        assert wtab is not None, f'Invalid Selection: {idx}'
+        return wtab.widget
+
     def wselection(self) -> str:
         '''Search for the current selected tab.
 
@@ -886,17 +1049,25 @@ class Notebook(ttk.Notebook, mixin.ContainerWidget):
         '''
         # TODO: Optimise this? Save a dict of indexes or something?
         selected_id = self.select()
-        if __debug__:
-            logger.debug('S: %r', selected_id)
-            tabs_id = [str(w) for w in self.tabs()]
-            logger.debug(' | %s', ' '.join(tabs_id))
+        # if __debug__:
+        #     logger.debug('S: %r', selected_id)
+        #     tabs_id = [str(w) for w in self.tabs()]
+        #     logger.debug(' | %s', ' '.join(tabs_id))
         for idx, wtab in self.wtabs.items():
             if str(wtab.widget) == selected_id:
                 return idx
         raise ValueError('{self!r}: Invalid current selection: {selected_id!r}')
 
-    def selection_set(self, idx: str):
-        '''Select a tab by identifier.'''
+    def wselect(self, idx: str) -> None:
+        '''Select a tab by identifier.
+
+        Args:
+            idx: The tab identifier.
+
+        Note:
+            In debug mode, this will fail if called with the wrong identifier.
+            This check is skipped for performance on optimized mode.
+        '''
         wtab = self.wtabs.get(idx, None)
         assert wtab is not None, f'Invalid Selection: {idx}'
         return self.select(tab_id=str(wtab.widget))
@@ -915,7 +1086,7 @@ class Notebook(ttk.Notebook, mixin.ContainerWidget):
             selected_idx = keys.index(selected)
             new_selected = keys[selected_idx - 1]
         if new_selected:
-            self.selection_set(new_selected)
+            self.wselect(new_selected)
 
     def _tabScrollDown(self, event=None):
         keys = list(self.wtabs.keys())
@@ -931,7 +1102,7 @@ class Notebook(ttk.Notebook, mixin.ContainerWidget):
             selected_idx = keys.index(selected)
             new_selected = keys[selected_idx + 1]
         if new_selected:
-            self.selection_set(new_selected)
+            self.wselect(new_selected)
 
 
 class NotebookUniform(Notebook):
@@ -1081,8 +1252,9 @@ class Tree(ttk.Treeview, mixin.SingleWidget):
         self.wdata = {}
         # Selection
         if selectable:
-            self.binding('<Double-Button-1>', lambda e: 'break')  # Disable double click
             self.binding('<<TreeviewSelect>>', self._onSelect)
+            # Disable Double-Click event, when selectable
+            self.binding('<Double-Button-1>', fn.binding_disable)
         # Appearance
         if expand:
             self.grid(sticky=tk.NSEW)
@@ -1156,6 +1328,7 @@ class Tree(ttk.Treeview, mixin.SingleWidget):
             #     tpl_text = f'{parent or "__top__"}::#{eid}'
             #     logger.debug(f'{">" * (_level + 1)} {tpl_text}: L:"{element.label}" C:|{" ".join(element.columns)}|')
             if element.columns is not None:
+                # TODO: Support a dict with keys corresponding to the `self.wcolumns`, possible subset
                 assert len(element.columns) == len(self.wcolumns), f'Invalid Column #{eid}: Size: E{len(element.columns)} W{len(self.wcolumns)}'
             cid = self.insert(parent or '', parent_loc,
                               text=element.label, values=element.columns or [],
@@ -1220,7 +1393,7 @@ class Tree(ttk.Treeview, mixin.SingleWidget):
         if __debug__:
             selectmode = str(self['selectmode'])
             assert selectmode in [tk.BROWSE, tk.NONE], f'{self!r}: Invalid Selection Mode: {selectmode}'
-            logger.debug('Selection: %r', self.selection())
+            # logger.debug('Selection: %r', selection)
         if len(selection) == 0:
             # Skip un-selections
             # Usually the Tree contents changed...
@@ -1419,3 +1592,67 @@ class Listbox(Tree):
                 if __debug__:
                     logger.debug(f'{self}: Auto Height: {wsize}')
                 self.configure(height=wsize)
+
+
+class FramePaned(ttk.PanedWindow, mixin.ContainerWidget):
+    '''A frame to hold other widgets, with user-controllable relative sizes.
+
+    This is similar to a `FrameUnlabelled`, restricted to horizontal or
+    vertical layouts, but where the interfaces between the children can be
+    adjusted by the user, to resize them at will.
+
+    This allows some child widgets to be hidden, by resizing the other panes to
+    fully occupy the widget area.
+
+    Note that using this with more than two children can be confusing for the
+    users, particularly when resizing takes place.
+
+    There is no Python documentation, see ``Tk`` `ttk.PanedWindow <https://www.tcl.tk/man/tcl/TkCmd/ttk_panedwindow.html>`_ documentation.
+
+    Args:
+        orient: The orientation of the child widgets. Restricted to
+            `tk.VERTICAL` or `tk.HORIZONTAL`.
+        layout: Alias for "orient".
+        weight: The weight for each children widget. Optional, defaults to 1.
+        weights: Per-child widget. Optional, defaults to the common "weight".
+
+    Note:
+        The "orient" argument has similar semantics to the rest of frames,
+        which means the opposite from the upstream ``ttk`` semantics.
+
+    See Also:
+        - `FrameUnlabelled`, `FrameLabelled`: Similar versions, without user
+          control over sizing.
+    '''
+    layout: typing.Optional[str] = None
+
+    __orientation: typing.Mapping[str, typing.Union[typing.Literal['vertical'], typing.Literal['horizontal']]] = {
+        tk.VERTICAL: tk.HORIZONTAL,
+        tk.HORIZONTAL: tk.VERTICAL,
+    }
+
+    def __init__(self, parent,
+                 orient: typing.Optional[str] = None,
+                 *args,
+                 weight: typing.Optional[int] = None,
+                 weights: typing.Optional[typing.Mapping[str, int]] = None,
+                 **kwargs):
+        # Orientation can be given as the layout
+        # But the ContainerWidget layout is always None
+        # No Default Orientation, must be given somewhere
+        # - Switch the meaning of the orientations around so that it's
+        #   consistent with the other container layouts.
+        orientation = orient or kwargs.pop('layout', None) or self.layout
+        assert orientation is not None and orientation in self.__orientation, f'Invalid Orientation: {orientation}'
+        super().__init__(parent, orient=self.__orientation[orientation])  # ttk.PanedWindow
+        self.init_container(*args, layout=None, **kwargs)
+        for wname, widget in self.widgets.items():
+            # Widget Weight: Default to 1
+            wweight = weight or 1
+            if weights:
+                # Per-Widget Override
+                wweight = weights.get(wname, wweight)
+            self.add(widget,
+                     weight=wweight)
+        logger.debug('%r', list(self.panes()))
+        assert len(self.panes()) == len(self.widgets)
