@@ -1,14 +1,16 @@
 import logging
 
+from pathlib import Path
+from functools import wraps
 from datetime import datetime
 import random
 
 import tkinter as tk
 from . import MODULES_VERBOSE
-from . import RootWindow, FrameUnlabelled, FrameLabelled, FrameStateful, Notebook, NotebookUniform
-from . import Button, Checkbox, Entry, Label, Listbox, Combobox, ComboboxMap, EntryMultiline, Tree
-from . import var, spec
-from .model import CP, BindingGlobal, Timeout, TimeoutIdle
+from . import RootWindow, FrameUnlabelled, FrameLabelled, FrameStateful, FramePaned, Notebook, NotebookUniform
+from . import Button, Checkbox, Entry, Label, LabelStateful, Listbox, Combobox, ComboboxMap, EntryMultiline, Tree
+from . import var, spec, fn
+from .model import CP, BindingGlobal, Timeout, TimeoutIdle, FileType, FileTypes
 
 
 logger = logging.getLogger(__name__)
@@ -46,14 +48,32 @@ class ListFrame_Outer(FrameStateful):
     layout = 'R2,1'
 
     def setup_widgets(self, *, cbox1):
-        self.left = ListFrame_Inner(self, label='Left', cvariableDefault=True)
-        self.right = ListFrame_Inner(self, label='Right', cvariableDefault=False)
+        self.left = ListFrame_Inner(self, label='Left',
+                                    cvariableDefault=True)
+        self.right = ListFrame_Inner(self, label='Right',
+                                     cvariableDefault=False)
         self.bottom = ListFrame_Inner(self, label='Bottom',
                                       cvariable=cbox1)
 
 
+class ListFrame_Lists(FramePaned):
+    layout = tk.VERTICAL
+
+    def setup_widgets(self, *, vLst):
+        self.lstS = Listbox(self,
+                            height=6,
+                            variable=vLst,
+                            selectable=True)  # Selectable
+        self.lstRO = Listbox(self, label='Unselectable',
+                             maxHeight=3, expand=True,  # Varying Height, Expanded  # N/A on FramePaned
+                             variable=vLst,
+                             selectable=False,
+                             style=Listbox.Style(altbg=True),
+                             )  # Not Selectable
+
+
 class ListFrame(FrameLabelled):
-    layout = 'R2,1,1,1,1,1'
+    layout = 'R2,1,2,1'
 
     def setup_widgets(self, *, cbox1):
         i_choices: spec.StaticMap[int] = spec.StaticMapValues(lambda i: 'I%d' % i, range(10), defaultValue=7)
@@ -61,16 +81,8 @@ class ListFrame(FrameLabelled):
         self.bFill = Button(self, label='Fill Me!')
         self.bCheck = Button(self, label='Check')
 
-        vLst = var.StringList(name='lst')  # Alternative: self.var(var.StringList, name='lst')
-        self.lstS = Listbox(self,
-                            variable=vLst,
-                            selectable=True)  # Selectable
-        self.lstRO = Listbox(self, label='Unselectable',
-                             maxHeight=3, expand=True,  # Varying Height, Expanded
-                             variable=vLst,
-                             selectable=False,
-                             style=Listbox.Style(altbg=True),
-                             )  # Not Selectable
+        vLst = self.var(var.StringList, name='lst')
+        self.cLst = ListFrame_Lists(self, vLst=vLst)
 
         self.bChoice = Button(self, label='CB=2')
         self.i_choice = ComboboxMap(self, values=i_choices)  # label='CB(int)'
@@ -79,7 +91,7 @@ class ListFrame(FrameLabelled):
                                          cbox1=cbox1)
 
     def setup_layout(self, layout):
-        self.rowconfigure(self.lstS.grid_info().get('row'), weight=0)
+        self.rowconfigure(self.cLst.grid_info().get('row'), weight=0)
 
     def setup_defaults(self):
         self.fill_lst()
@@ -99,10 +111,10 @@ class ListFrame(FrameLabelled):
 
     def fill_lst(self):
         ctime = str(datetime.now())
-        self.lstS.wstate = ['A', 'List', 'Of', 'Letters', '@', ctime]
+        self.gvar('lst').set(['A', 'List', 'Of', 'Letters', '@', ctime])
 
     def check_lst(self):
-        sel = self.lstS.wselection()
+        sel = self.cLst.lstS.wselection()
         logger.debug('S: %r', sel)
 
     def onChosen(self, variable, etype, *, spec):
@@ -133,9 +145,24 @@ class NB_Child_Simple(FrameUnlabelled):
         w = {}
         for n in range(5):
             w[f'n{n}'] = Label(self, label=f'{n}: {label}')
-        w['e'] = Entry(self)  # label=f'Entry #{n}'
+        w['e'] = LabelStateful(self)  # label=f'LS #{n}'
         return w
 
+    def setup_defaults(self):
+        self.widgets['e'].onClick = self.onClick_E
+
+    def setup_adefaults(self):
+        self.widgets['e'].wstate = 'Clickable LabelStateful'
+
+    def onClick_E(self, event=None):
+        w = self.widgets['e']
+        state = w.wstate
+        if not state.endswith(' T'):
+            state += ' T'
+        else:
+            state = state[:-2]
+        w.wstate = state
+
 
 class NB_Child_Complex(NotebookUniform):
     tabids = {f'TC{d}': f'Tab Complex {d}' for d in range(5)}
@@ -300,17 +327,63 @@ class NB_Child_Timeouts(FrameUnlabelled):
         self.timeout_idle_c = NB_Child_TimeoutIdle_Chain(self)
 
 
+class NB_Child_Dialog(FrameUnlabelled):
+    layout = 'Rx,1'
+
+    def setup_widgets(self):
+        self.ds = Button(self, label='D S')
+        self.dl = Button(self, label='D L')
+        self.fs = Button(self, label='F S')
+        self.fl = Button(self, label='F L')
+        self.flc = Button(self, label='F L(py)')
+        self.fsc = Button(self, label='F S(py)')
+        self.txt = LabelStateful(self)
+
+        self.ds.onClick = self.click(fn.ask_directory_save, self, title='Directory @ .',
+                                     initialDirectory=Path('.'))
+        self.dl.onClick = self.click(fn.ask_directory_load, self, title='Directory @ Home',
+                                     initialDirectory=Path('~').expanduser())
+        self.fs.onClick = self.click(fn.ask_file_save, self, title='File @ ..',
+                                     initialDirectory=Path('..'))
+        self.fl.onClick = self.click(fn.ask_file_load, self, title='File @ /',
+                                     initialDirectory=Path('/'))
+        self.fsc.onClick = self.click(self.customFile, fn.ask_file_save)
+        self.flc.onClick = self.click(self.customFile, fn.ask_file_load)
+
+    def click(self, fn, *args, **kwargs):
+        @wraps(fn)
+        def wrapped():
+            self.txt.wstate = ''
+            ret = fn(*args, **kwargs)
+            ret_loc = ret.resolve() if ret else ret
+            ret_exists = str(ret.exists()) if ret else 'N/A'
+            self.txt.wstate = f'{ret_loc}\nExists: {ret_exists}'
+        return wrapped
+
+    def customFile(self, function):
+        return function(self, title='Custom Python Files @ .',
+                        initialDirectory=Path('.'),
+                        includeAll=False, filetypes=FileTypes({
+                            'Python': FileType('py'),
+                            'TOML': FileType('toml'),
+                        }))
+
+
 class NB(Notebook):
     def setup_tabs(self):
         logger.debug('» %r', self)
         # TODO: Test images, etc.
         return {
             'tt': Notebook.Tab('Timeouts', NB_Child_Timeouts(self)),
+            'td': Notebook.Tab('Dialogues', NB_Child_Dialog(self)),
             'tc': Notebook.Tab('Tab Complex', NB_Child_Complex(self)),
             't1': Notebook.Tab('Tab 1', NB_Child_Simple(self, label='Tab 1')),
             't2': Notebook.Tab('Tab 2', NB_Child_Simple(self, label='Tab 2')),
         }
 
+    def setup_adefaults(self):
+        self.wselect('td')
+
 
 class TextEditor(FrameStateful):
     label = 'LTML'
@@ -434,9 +507,12 @@ class RW(RootWindow):
         logging.info('=> State @ %s[%r]', self, self)
         for line in pformat(self.wstate).splitlines():
             logging.info('%s', line)
-        logging.info('=> State @ ubox')
-        for line in pformat(self.ubox.wstate).splitlines():
-            logging.info('%s', line)
+        # logging.info('=> State @ ubox')
+        # for line in pformat(self.ubox.wstate).splitlines():
+        #     logging.info('%s', line)
+        # logging.info('=> State @ lf.cLst')
+        # for line in pformat(self.lf.cLst.wstate).splitlines():
+        #     logging.info('%s', line)
         logging.info('=> State Set')
         new = self.wstate
         for b in ('0', '12'):
