@@ -170,7 +170,7 @@ class MixinStateContainer(MixinState):
     '''
     wstate_static: bool = False
 
-    def state_get(self, *args, **kwargs):
+    def state_get(self, **kwargs):
         state = {}
         for identifier, wds in self.stateSetup(**kwargs).items():
             result = wds.getter()
@@ -516,6 +516,9 @@ class ContainerWidget(MixinWidget, MixinStateContainer):
         self.setup_defaults()
         self.after_idle(lambda: self.setup_adefaults())  # No need for a `TimeoutIdle` here
         assert hasattr(self, 'grid'), f'{self!r} should have a grid method'
+        if __debug__:
+            aliases = set(self._variables.keys()).intersection(set(self.widgets.keys()))
+            assert len(aliases) == 0, f'{self!r}: Aliased var/widgets: {" ".join(aliases)}'
 
     def setup_widgets(self, *args, **kwargs) -> typing.Optional[typing.Mapping[str, MixinWidget]]:
         '''Define the sub widgets here.
@@ -525,8 +528,18 @@ class ContainerWidget(MixinWidget, MixinStateContainer):
         raise NotImplementedError
 
     def var(self, cls: typing.Type[var.Variable], *, value=None, name=None) -> tk.Variable:
-        '''
-        "Attach" a new variable to this container.
+        '''"Attach" a new variable to this container.
+
+        Args:
+            cls: The variable class.
+
+            value: The default value.
+                Optional, defaults to `None`.
+            name: The variable name.
+                Optional, defaults to an autogenerated name.
+
+        See Also:
+            gvar: Access the variable by name.
         '''
         vobj = cls(value=value)
         assert isinstance(vobj, var.Variable), f'Class "{cls}" is not a "tk.Variable"'
@@ -534,6 +547,19 @@ class ContainerWidget(MixinWidget, MixinStateContainer):
         self._variables[name or str(vobj)] = vobj
         return vobj
 
+    def gvar(self, name: str):
+        '''Get a variable attached to this container, by name.
+
+        Fails if it does not exist.
+
+        Args:
+            name: The variable name to search for.
+
+        See Also:
+            var: Attach a variable to this container, with a name.
+        '''
+        return self._variables[name]
+
     def layout_container(self, layout: typing.Optional[str], expand: bool, autogrow: bool):
         assert isinstance(self, (tk.Widget, tk.Tk)), f'{self} is not a valid tkinter.Widget'
         if expand:
@@ -657,7 +683,7 @@ class ContainerWidget(MixinWidget, MixinStateContainer):
         '''GUI grid size (according to the current child widgets).'''
         return fn.grid_size(*self.widgets.values())
 
-    def state_c(self, *, vid_upstream: typing.Set[str] = None) -> ContainerState:
+    def state_c(self, *, vid_upstream: typing.Optional[typing.Set[str]] = None) -> ContainerState:
         swidgets = {}
         cwidgets = {}
         wvariables = {}
@@ -702,10 +728,12 @@ class ContainerWidget(MixinWidget, MixinStateContainer):
         for v, ws in container_state.vwidgets.items():
             if v is not None and len(ws) > 1:
                 wv = container_state.wvariables[v]
+                assert v not in rvalue, f'{self!r}: Aliased vwidgets "{v}"'
                 rvalue[v] = model.WidgetDynamicState(wv.get, wv.set, noneable=False)
                 wids_done.extend(ws)
         for n, w in container_state.swidgets.items():
             if n not in wids_done:
+                assert n not in rvalue, f'{self!r}: Aliased swidgets "{n}"'
                 rvalue[n] = model.WidgetDynamicState(
                     w.wstate_get,
                     w.wstate_set,
@@ -713,6 +741,7 @@ class ContainerWidget(MixinWidget, MixinStateContainer):
                 )
         vid_upstream = container_state.vid_upstream
         for n, wc in container_state.cwidgets.items():
+            assert n not in rvalue, f'{self!r}: Aliased cwidgets "{n}"'
             rvalue[n] = model.WidgetDynamicState(
                 partial(wc.state_get, vid_upstream=vid_upstream),
                 partial(wc.state_set, vid_upstream=vid_upstream),
