@@ -7,7 +7,9 @@ import warnings
 from dataclasses import dataclass, field as dc_field, fields, astuple as dc_astuple, replace as dc_replace, asdict as dc_asdict, InitVar
 import enum
 import abc
+from functools import cached_property
 import math
+from pathlib import Path
 from fractions import Fraction
 import tkinter as tk
 import typing
@@ -17,6 +19,7 @@ from . import util
 if typing.TYPE_CHECKING:
     # Avoid circular imports
     from . import mixin
+    from . import EntryMultiline
 
 
 vsT = typing.TypeVar('vsT')
@@ -33,6 +36,35 @@ logger = logging.getLogger(__name__)
 logger_layout = logging.getLogger('%s.layout' % __name__)
 
 
+class FileType(typing.Tuple[str]):
+    def __new__(cls, *args: str):
+        true_args = (f'.{s}' for s in args)
+        return super().__new__(cls, true_args)  # type: ignore
+
+    def matches(self, path: Path):
+        if __debug__:
+            suff = tuple(path.suffixes[-len(self):])
+            res = '==' if self == suff else '<=>'
+            logger.debug(f'F[{path}]={suff} {res} {self}')
+        return path.name.endswith(self.suffix)
+
+    @cached_property
+    def suffix(self):
+        return ''.join(self)
+
+    @cached_property
+    def pattern(self):
+        return f'*{self.suffix}'
+
+
+class FileTypes(typing.Dict[str, FileType]):
+    def allbut(self, *keys: str):
+        return {k: v for k, v in self.items() if k not in keys}
+
+    def only(self, *keys: str):
+        return {k: self[k] for k in keys}
+
+
 @dataclass
 class WStyle:
     '''Widget style object.
@@ -397,10 +429,11 @@ class WState(typing.Generic[wsT, wsST]):
     This is needed to mark state as being a product of a wrapped container.
     '''
 
-    state: wsT
+    state: typing.Optional[wsT]
     '''The widget state.
 
     This is the extra value being stored on the widget.
+    Optional, if the variable already exists upstream.
     '''
     substate: wsST
     '''The wrapped state.
@@ -502,7 +535,62 @@ assert set(GUI_STATES.keys()) == set(f.name for f in fields(GuiState))
 assert set(GUI_STATES_common) < set(f.name for f in fields(GuiState))
 
 
-class Binding:
+class MixinBinding:
+    ''''''  # Internal, do not document
+    _binding: typing.Callable
+
+    def __init__(self, widget: 'mixin.MixinWidget', sequence: str, action: typing.Callable, *, immediate: bool = False, description: typing.Optional[str] = None):
+        assert isinstance(widget, (tk.Widget, tk.Tk)), f'{widget} is not a valid widget'
+        self.obj = None
+        self.widget = widget
+        self.sequence = sequence
+        self.action = action
+        self.description: str = description or f'Calling "{action.__name__}"'
+        if immediate:
+            self.enable()
+
+    def __bool__(self):
+        '''Check if the binding is enabled.
+
+        Should be used as:
+
+        .. code:: python
+
+            if binding:
+                print('Binding is enabled')
+            else:
+                print('Binding is disabled')
+        '''
+        return self.obj is not None
+
+    def _bind(self) -> typing.Any:
+        raise NotImplementedError
+
+    def _unbind(self, obj: typing.Any) -> None:
+        raise NotImplementedError
+
+    def enable(self) -> None:
+        '''Enable the binding.'''
+        if self.obj is None:
+            self.obj = self._bind()
+        else:
+            warnings.warn(f'Redundant enable @ {self!r}')
+
+    def disable(self) -> None:
+        '''Disable the binding.'''
+        if self.obj is not None:
+            obj = self.obj
+            try:
+                self.obj = None
+                self._unbind(obj)
+            except Exception as e:
+                self.obj = obj  # Restore
+                raise Exception(f'Error when unbinding @ {self.widget!r}') from e
+        else:
+            warnings.warn(f'Redundant disable @ {self!r}')
+
+
+class Binding(MixinBinding):
     '''Wrap a stateful binding.
 
     Can be enabled and disabled separately from creation. Default to starting
@@ -527,69 +615,35 @@ class Binding:
         description: Optional description of the binding action.
             Useful for debugging, not used for any purpose for now.
 
+    Note:
+        When the ``action`` function returns ``"break"``, no more actions are
+        considered. Use this with care.
+
     See Also:
         For the global version of this, see `BindingGlobal`.
 
     .. automethod:: __bool__
+    .. automethod:: enable
+    .. automethod:: disable
     '''
-    def __init__(self, widget: 'mixin.MixinWidget', sequence: str, action: typing.Callable, *, immediate: bool = False, description: typing.Optional[str] = None, _bind_all: bool = False):
-        assert isinstance(widget, (tk.Widget, tk.Tk)), f'{widget} is not a valid widget'
-        # `_bind_all` is not documented on purpose, use `BindingGlobal` always.
-        self.obj = None
-        self.widget = widget
-        self.sequence = sequence
-        self.action = action
-        self._fun = {
-            False: self.widget.bind,
-            True: self.widget.bind_all,
-        }[_bind_all]
-        if _bind_all is False and sequence in widget.wroot._bindings_global:
+    def __init__(self, widget: 'mixin.MixinWidget', sequence: str, *args, **kwargs):
+        if sequence in widget.wroot._bindings_global:
             # Warn when widget bindings are aliased by global bindings
             warnings.warn(f'Global binding for "{sequence}" aliases {widget!r} binding')
             # Should this be an error?
-        self.description: str = description or f'Calling "{action.__name__}"'
-        if immediate:
-            self.enable()
+        super().__init__(widget, sequence, *args, **kwargs)
 
-    def __bool__(self):
-        '''Check if the binding is enabled.
+    def _bind(self):
+        return self.widget.bind(sequence=self.sequence, func=self.action, add=True)
 
-        Should be used as:
+    def _unbind(self, obj):
+        self.widget.unbind(self.sequence, obj)
 
-        .. code:: python
-
-            if binding:
-                print('Binding is enabled')
-            else:
-                print('Binding is disabled')
-        '''
-        return self.obj is not None
-
-    def enable(self):
-        '''Enable the binding.'''
-        if self.obj is None:
-            self.obj = self._fun(self.sequence, self.action, add=True)
-        else:
-            warnings.warn(f'Redundant enable @ {self!r}')
-
-    def disable(self):
-        '''Disable the binding.'''
-        if self.obj is not None:
-            obj = self.obj
-            try:
-                self.obj = None
-                self.widget.unbind(self.sequence, obj)
-            except Exception as e:
-                self.obj = obj  # Restore
-                raise Exception(f'Error when unbinding @ {self.widget!r}') from e
-        else:
-            warnings.warn(f'Redundant disable @ {self!r}')
-
-    def __repr__(self):
-        return '%s("%s" @ %s)  # %s' % (self.__class__.__name__, self.sequence, self.widget, self.description)
+    def __repr__(self) -> str:
+        return f'Binding("{self.sequence}" @ {self.widget})  # {self.description}'
 
 
-class BindingGlobal(Binding):
+class BindingGlobal(MixinBinding):
     '''Wrap a stateful global binding.
 
     This is a global version of the `Binding` object. The main difference is
@@ -600,16 +654,91 @@ class BindingGlobal(Binding):
     The `RootWindow` will store a dictionary of sequences to `BindingGlobal`.
     This is similar to `mixin.MixinWidget.binding`.
 
-    See `Binding` for usage.
+    Args:
+        widget: The widget to apply the binding to. This can be any widget,
+            only the corresponding `RootWindow` is considered.
+        sequence: The sequence to bind to. See ``Tk`` `bind
+            <https://www.tcl.tk/man/tcl/TkCmd/bind.html>`_ documentation, not
+            everything applies here.
+        action: The function to call when the binding is hit.
+        immediate: Enable binding on creation.
+            Defaults to ``False``, requiring separate activation.
+        description: Optional description of the binding action.
+            Useful for debugging, not used for any purpose for now.
+
+    Note:
+        When the ``action`` function returns ``"break"``, no more actions are
+        considered. Use this with care.
+
+    See Also:
+        For the widget version of this, see `Binding`.
+
+    .. automethod:: __bool__
+    .. automethod:: enable
+    .. automethod:: disable
     '''
     def __init__(self, widget: 'mixin.MixinWidget', sequence: str, *args, **kwargs):
         root_widget = widget.wroot
         if sequence in root_widget._bindings_global:
             raise ValueError(f'Repeated global binding for "{sequence}"')
-        super().__init__(root_widget, sequence, *args, _bind_all=True, **kwargs)
+        super().__init__(root_widget, sequence, *args, **kwargs)
         # Store all global bindings
         root_widget._bindings_global[sequence] = self
 
+    def _bind(self):
+        return self.widget.bind_all(sequence=self.sequence, func=self.action, add=True)
+
+    def _unbind(self, obj):
+        self.widget.unbind(self.sequence, obj)
+
+    def __repr__(self) -> str:
+        return f'BindingGlobal("{self.sequence}")  # {self.description}'
+
+
+class BindingTag(MixinBinding):
+    '''Wrap a stateful tag binding for `EntryMultiline`.
+
+    This is a version of the `Binding` object which applies only to the tags in
+    `EntryMultiline`.
+
+    Args:
+        widget: The widget to apply the binding to. This can be any widget,
+            only the corresponding `RootWindow` is considered.
+        tag: The sub-widget tag where the binding applies.
+        sequence: The sequence to bind to. See ``Tk`` `bind
+            <https://www.tcl.tk/man/tcl/TkCmd/bind.html>`_ documentation, not
+            everything applies here.
+        action: The function to call when the binding is hit.
+        immediate: Enable binding on creation.
+            Defaults to ``False``, requiring separate activation.
+        description: Optional description of the binding action.
+            Useful for debugging, not used for any purpose for now.
+
+    Note:
+        When the ``action`` function returns ``"break"``, no more actions are
+        considered. Use this with care.
+
+    .. automethod:: __bool__
+    .. automethod:: enable
+    .. automethod:: disable
+    '''
+    def __init__(self, widget: 'EntryMultiline', tag: str, sequence: str, *args, **kwargs):
+        self.tag = tag
+        if sequence in widget.wroot._bindings_global:
+            # Warn when widget bindings are aliased by global bindings
+            warnings.warn(f'Global binding for "{sequence}" aliases {widget!r}(tag "{tag}") binding')
+            # Should this be an error?
+        super().__init__(widget, sequence, *args, **kwargs)
+
+    def _bind(self):
+        return self.widget.tag_bind(tagName=self.tag, sequence=self.sequence, func=self.action, add=True)
+
+    def _unbind(self, obj):
+        self.widget.tag_unbind(self.tag, self.sequence, obj)
+
+    def __repr__(self) -> str:
+        return f'BindingTag("{self.sequence}" @ {self.widget}|{self.tag})  # {self.description}'
+
 
 class Timeout:
     '''Schedule an action into the future time, cancellable.
@@ -976,6 +1105,8 @@ class TextElement_span(TextElement):
                     raise ValueError(f'Invalid tag complex: {self.tag} @ {self.tags}')
         else:
             assert self.tags is None, 'Secondary tags without main tag'
+        if '\n' in self.text:
+            warnings.warn('<span> should not have newlines, use <br/>', stacklevel=14)
         self.has_tags = True
 
     @property
